function [lmainindex,rmainindex,ldata,ldist,rdata,rdist,cluster,no,centroid] = split2(data,dist,a,mainindex,scale,mnp,percentage,cluster,no,centroid)

xleftindexes = find(dist <= a);          %Finding the indexes of points whose distance value is less than equal to cutoff value
xrightindexes = find(dist > a);          %Finding the indexes of points whose distance value is more than the cut off value    

lmainindex = mainindex(xleftindexes,:);  %Returns the corresponding indexes of the main dataset 
rmainindex = mainindex(xrightindexes,:); 
totalindex = [xleftindexes; xrightindexes]; %Returns the indexes of left and right as one whole matrix

ldata = data(xleftindexes,:);            %Returns the data that are to the left of cut off point 

if size(ldata,1) ~= 0
ldist = dist(xleftindexes,:);            %Returns the distances left of cut off point

meanl = findDistMean(ldist, ldata(:, 4));                     %Mean of the left distances  
[indexl] = ptclosest(ldist,meanl);       %Finds the data point which is closest to the mean which is the centroid

point_meanl = findMean(ldata);      %Mean of points   
[point_indexl] = dataptclosest(ldata,point_meanl); %Finds the data point which is closest to the mean 

end

rdata = data(xrightindexes,:);           %Returns the data that are to the right of cut off point 

if size(rdata,1) ~= 0
rdist = dist(xrightindexes,:);           %Returns the distances right of cut off point

meanr = findDistMean(rdist, rdata(:, 4));                     %Mean of the right distances   
[indexr] = ptclosest(rdist,meanr);       %Finds the data point which is closest to the mean which is the centroid

point_meanr = findMean(rdata);      %Mean of points 
[point_indexr] = dataptclosest(rdata,point_meanr);  %Finds the data point which is closest to the mean

end

sizeldata = size(ldata,1);
sizerdata = size(rdata,1);

if sizeldata == 0 & sizerdata ~= 0  %If while splitting one side has no data the other side is taken as
    sprintf('%s','Done')            %one cluster  
    cluster(rmainindex,:) = no;
    no = no + 1;                     %This takes care of the cluster numbers
    centroid(rmainindex,:) = rmainindex(point_indexr,:); %Centroid
    
    lmainindex = 0; rmainindex = 0; ldist = 0; rdist = 0;
    return
elseif sizeldata ~= 0 & sizerdata == 0  %If while splitting one side has no data the other side is taken as
    sprintf('%s','Done')                 %one cluster
    cluster(lmainindex,:) = no;
    no = no + 1;                     %This takes care of the cluster numbers
    centroid(lmainindex,:) = lmainindex(point_indexl,:); %Centroid
    
    lmainindex = 0; rmainindex = 0; ldist = 0; rdist = 0;
    return
end

j = 1;
for i=1:size(ldata,1)                    %Computes the distances of all the data points from the left centroid
    distance(j,1) = dist2pt(ldata(i,:), ldata(indexl,:)); 
    j = j+1;
end

for i=1:size(rdata,1)
    distance(j,1) = dist2pt(rdata(i,:), ldata(indexl,:));
    j=j+1;    
end

j = 1;                                   %Computes the distances of all the data points from the right centroid
for i=1:size(ldata,1)
    distance(j,2) = dist2pt(ldata(i,:), rdata(indexr,:));
    j = j+1;   
end

for i=1:size(rdata,1)
    distance(j,2) = dist2pt(rdata(i,:), rdata(indexr,:));
    j = j+1;
end

distance_centroid = dist2pt(ldata(indexl,:), rdata(indexr,:)); %Distances between the centroids

j = 1;
k = 1;
for i=1:size(distance,1)    %Split performed according to the condition the data points satisfying
    if (distance(i,1) - distance(i,2)) > percentage*distance_centroid  % the condition d(x,c1) - d(x,c2) > d(c1,c2)
        splitleft(j,:) = i;                              %forms the left side of the split
        j = j + 1;          %Data points satisfying the condition  d(x,c2) - d(x,c1) > d(c1,c2) forms the right split                         
    elseif (distance(i,2) - distance(i,1)) > percentage*distance_centroid
        splitright(k,:) = i;   %all the other datapoints are considered outliers
        k = k + 1;     0
    end
end

lmainindex = 0;
rmainindex = 0;

ldata = 0;
ldist = 0;
ldata = data(totalindex(splitleft,:),:);            %Returns the data that are to the left 

size_ldata = size(ldata,1);
if size(ldata,1) > mnp
    lmainindex = mainindex(totalindex(splitleft,:),:);
    ldist = dist(totalindex(splitleft,:),:);            %Returns the distances that are to the left
% else
%     ldata = 0;
end

rdata = 0;
rdist = 0;
rdata = data(totalindex(splitright,:),:);           %Returns the data that are to the right 

if size(rdata,1) > mnp
    rmainindex = mainindex(totalindex(splitright,:),:);
    rdist = dist(totalindex(splitright,:),:);           %Returns the distances that are to the right
% else
%     rdata = 0;
end


if size(ldata,1) <= mnp & size(rdata,1) <= mnp    %If the number of points in both left and right split
    lmainindex = mainindex(totalindex(splitleft,:),:); %is less than minimum number of points
    rmainindex = mainindex(totalindex(splitright,:),:); %no split done all datapoints are taken as one 
    ldist = dist(totalindex(splitleft,:),:);            %cluster
    rdist = dist(totalindex(splitright,:),:); 
    
    total_lr_index = [lmainindex; rmainindex];     %Merging all the indexes
    total_lr_dist = [ldist; rdist];                %merging all the distances
    total_lr_data = [ldata; rdata];                %merging all the data
    
    mean_lr_data = findMean(total_lr_data);      %Mean of data  
    [index_lr] = dataptclosest(total_lr_data,mean_lr_data);  %datapoint closest to the mean
    
    sprintf('%s','Done')             
    cluster(total_lr_index,:) = no;
    no = no + 1;                     %This takes care of the cluster numbers
    centroid(total_lr_index,:) = total_lr_index(index_lr,:);
    
    lmainindex = 0; rmainindex = 0; ldist = 0; rdist = 0;
end
        