%Clustering algorithm by Longin Jan Latecki dt: 10/02/02
%Written by Qifang Xu
%data clustering algorithm in color space

%firstauto2(inputFilename,mnp,percentage)
%input:
%inputFilename: Text file, each row contains a data vector of the data to be clustered
%mnp          : Minimal number of points in each cluster
%percentage   : Parameter that delays clustering of points in a margin region (normal value .05 i.e. 5 percent)
%output:
%cluster      : A list of cluster indices assigned to data vectors

%example:
%
%[cluster] = firstauto1('cherry.jpg',30,0.07);
%[cluster] = firstauto1('noise_2.gif',3,0.01);

function [cluster] = firstauto3(inputFileName,mnp,percentage) %Call function by giving input file name
                                              %minimum number of points and percentage
 
data_rgb = imread(inputFileName);
info = imfinfo(inputFileName);
if strcmp(info.ColorType, 'truecolor')
    [data_ind, map] = rgb2ind(data_rgb, 256);
else
    map = info.ColorTable;
end
ddata = double(data_ind) + 1;  %Because of the offset, you must add 1 to convert a uint8 or uint16 indexed image to double.

% get the statistics of the image
[colorWeights] = adjustStat(ddata, map);

data = colorWeights(:, 1:4);

[row, col] = size(data);
% dataR(:, 1) = data(:, 1);
% dataR(:, 2) = data(:, col)
% dataG(:, 1) = data(:, 2);
% dataG(:, 2) = data(:, col)
% dataB(:, 1) = data(:, 3);
% dataB(:, 2) = data(:, col)

[mn, s, d] = normalize(data, [], []);    %normalize the dataset

%data(:, 1:3) = d;
x = data(:, 1:3);

for i=1:size(x,1)
    dist(i,1)= dist2pt(data(i, :), mn);  %Euclidean Distances of each point in the dataset from the mean
end


maxdist = max(dist);            %Finding the maximum distance from all the distances computed 
scalestep=maxdist/(size(dist,1)*0.66); %scalestep for the histogram
scale = 0:scalestep:maxdist;         %X axis scale for the histogram

for i=1:(size(x,1))
    mainindex(i,1) = i;         %Dummy matrix that has values from 1 to size of dataset in the format 1,2,3,.. for 
end                             %indexing purposes

cluster(1:size(x,1), 1) = 0;                    %Initialization
centroid(1:size(x,1), 1) = 0;
value = 0;
no = 1;                   

[cluster, no, centroid] = recurauto1(data,dist,scale,mainindex,cluster,no,centroid,value,mnp,percentage); %Calling recursive function

totalnoofclusters =  no - 1

for i = 1:totalnoofclusters         %Check to see how many points are in the cluster, before mapping
    pts_in_cluster_before_reallocation = size(cluster(find(cluster == i),:),1); %unclustered data points
end

pts_not_clustered = size(cluster(find(cluster == 0),:),1)  %Points left unclustered

if size(centroid(find(cluster == 0)),1) ~= 0      %Distances of all the unclustered datapoints from the 
    for i=1:totalnoofclusters                         %centroids are found and the datapoints are mapped to the
        temp = 0;                                     %closest cluster 
        temp = centroid(find(cluster==i));
        cl = 0;
        y = x(find(cluster == 0),:);
        for j=1:size(y,1)
            distance(i,j) = dist2pt(y(j,:), x(temp(1),:));
        end
    end

    [mini, pos] = min(distance);

    position = find(cluster == 0);

    if size(distance,1) == 1
        pos(1:size(position,1)) = 1;
    end

    for i = 1:size(position,1)
        temp = centroid(find(cluster == pos(1,i)),:);
        centroid(position(i,1),:) = temp(1);       %Computing centroids for unclustered data points
    end
    cluster(find(cluster == 0),:) = pos';      %Remapping of unclustered data points
end

total_no_of_pts_in_all_clusters = 0;          %Computes the total datapoints in all clusters
for i = 1:totalnoofclusters
    total_no_of_pts_in_all_clusters = total_no_of_pts_in_all_clusters + size(cluster(find(cluster == i),:),1);
end

total_no_of_pts_in_all_clusters

sdata(size(ddata))= 255;
% reconstruct the image
for i = 1:totalnoofclusters
    tmp = find(cluster == i);
    thisColorCentroid = centroid(tmp(1));
    %clusterBeforeColors = colorWeights(tmp, 1);
    for j = 1:length(tmp)
        [r, c] = find(ddata == tmp(j));
        for m = 1:length(r)
            sdata(r(m), c(m)) = thisColorCentroid;
        end
    end
    segData = uint8(sdata-1);
    figure, imshow(segData, map);
end

figure; imshow(data_ind, map);
segData = uint8(sdata-1);
figure, imshow(segData, map);

return;

    
        
        



    
